
;********************************************************************************
;* rubyStartup.s:
;*	System specific startup and library code for GIBL running under
;*	rubyOs
;********************************************************************************


;********************************************************************************
;* This file is part of gibl:							*
;*	Gordons Interactive Basic Language                                      *
;********************************************************************************
;*    A Tiny Basic for the 6502 inspired by the NIBL Tiny Basic interpreter;	*
;*    "National Industrial Basic Language" originally for the INS8060 SC/MP	*
;*    system.									*
;*										*
;*    gibl is distributed under a "Source Available" license.			*
;*	It is NOT Open source and must not be treated as such.			*
;*										*
;*    See the file LICENSE for details.						*
;*										*
;*    gibl is Copyright (c) 2023 by Gordon Henderson				*
;********************************************************************************


;********************************************************************************
;* Note of the Ruby Memory Map:
;*	It follows the Acorn MOS one relatively closely in that some areas are
;*	reserved - brifly:
;*
;* $0000-$008F	- User ZP storage
;* $0090-$00FF	- Reserved for Ruby OS
;* $0100-$01FF	- 6502 Stack
;* $0200-$02FF	- OS Variables, indirection vectors, etc.
;* $0300-$03FF	- keyboardIn - Used as the getline input buffer.
;* $0400-$07FF	- Reserved for the current "Language"
;* $0800-$0DFF	- Reserved for Ruby OS (input history, other buffers, etc.)
;* $0E00-$7FFF	- Application/Language workspace
;* $8000-$BFFF	- Application/Language code
;* $C000-$FDFF	- RubyOS
;* $FE00-$FEFF	- 65c22 VIA
;* $FF00-$FFFF	- IO area with host MCU and hardware vectors.
;********************************************************************************

	.include	"../rubyOs816/osVectors.h"

;********************************************************************************
;* External symbols you need to define:
;*	lineInput
;*     This is where the line if text is stored on keyboard input.
;*
;* External routines you need to define/re-define:
;*	putChar  - Print character in A.
;*	newLine  - Output a CR+LF
;*	putStr	 - Output a string in-line with progam code.
;*	getChar	 - Return character in A.
;*	getLine	 - Read in a line of text
;*			Line is CR terminated and located at lineInput.
;*	checkInt - Check for Ctrl-C or whatever. Set carry if Ctrl-C
;*
;*	In-General you need to preserve X and Y with data passes in/out in A.
;*
;* Other things to do:
;*	Make sure the 6502 is in the right mode (binary, not decimal),
;*	Set the stack appropriately (GIBL only uses a few JSRs, but the
;*		called IO routines may use more)
;*	Work out the start of RAM and store this in osPageL and osPageH
;*	Initialise IO, UARTs, whatever - if needed.
;*	Finally:
;*
;*		jmp gibl
;*
;********************************************************************************

; Create/Define lineInput:
;	We're using the botton of the stack page, so we must make sure we
;	initialise the stack high and leave plenty of space.

lineInput	= $0100

USE_OSWORD	:=	1


;********************************************************************************
;* Acorn/Ruby MOSify the start...
;*	This MAY not work as-is in a real BBC Micro. It may need work to
;*	make it run correcly as a sideways ROM in a BBC Micro unless you
;*	remove the BASIC ROM first...
;********************************************************************************

rom:	jmp	rubyStart				; Language entry
	jmp	rubyStart				; Service entry
	.byte	$41					; ROM type (BASIC)
here:	.byte	<copyr-1
	.byte	$1					; Version
	.asciiz	"GIBL"					; Name
copyr:	.byte	"(C) 2023 Gordon Henderson",13,10,0	; Copyright


;********************************************************************************
;* putChar:
;*	Print a single character in A. Preserve X & Y.
;********************************************************************************

putChar:	jmp	osWrch

;********************************************************************************
;* newLine:
;*	Output a newline to the terminal. Preserve X & Y.
;********************************************************************************

newLine:	jmp	osNewl


;********************************************************************************
;* getchar:
;*	Wait-for and read a single character back into A.
;********************************************************************************

getChar:	jmp	osRdch

;********************************************************************************
;* getLine:
;*	Read in a line of text.
;********************************************************************************

.ifdef	USE_OSWORD
	.include	"oswordGetline.s"
.else
	.include	"cheapGetline.s"
.endif


;********************************************************************************
;* putStr:
;*	Print a zero-terminated string stored in-line with the program code.
;*	Standard in RubyOS
;*	Usage:
;*		jsr	putStr
;*		.byte	"Hello, world", 0
;********************************************************************************

putStr:	jmp	strout


;********************************************************************************
;* checkInt:
;*	Check for a keyboard interrupt. This routine will be called
;*	regularly during program execution and listing
;********************************************************************************

.proc	checkInt
	lda	$FF
	bne	gotInt
	rts

gotInt:
	lda	#0
	sta	$FF
	pla			; Remove JSR
	pla
	lda	#eBRK
	jmp	ERR1
.endproc


;********************************************************************************
;* Start here.
;*	Note: RubyOS starts programs in 65C02 emulation mode.
;********************************************************************************

rubyStart:
	cld			; Paranoia

; We don't need much stack, so can use the start of page 1 as the keyboard
;	input buffer if needed.
;	This value is chosen more for debugging reasons than anything else.
;	but using this value suggests the RubyOS uses some 28 bytes in
;	normal use and running GIBL under RubyOS seems about the same...

	ldx	#$DF
	txs

;********************************************************************************
;* Program text start location.
;*	Programs start here and grow up towards the end of RAM.
;*	There is (currently) no check that RAM is exceeded.
;*
;*	Strings start at the end of program text and the built-in
;*	variable TOP will return that address.
;*
;* On non-acorn-ish systems, fix as required. 
;* Get PAGE and store in a known place so we can refer to it later in the code.
;*	We know it's $0E00 in RubyOS, but if someone ports it to a Beeb it may
;*	well be different so use osByte 131.
;********************************************************************************

	lda	#131
	jsr	osByte
	stx	osPageL
	sty	osPageH

; The main system initialisation and run-time.

	jmp	gibl


;********************************************************************************
;* rubyCommandCheck:
;*	See the input line is a ruby command we know about. Return 0 if handled.
;********************************************************************************

.proc	rubyCommandCheck
	lda	lineInput
	cmp	#'*'			; Star command
	bne	:+
	ldx	#<lineInput
	ldy	#>lineInput
	jsr	osCli
	lda	#0
	rts

:	cmp	#'#'			; Reload
	bne	:+
	ldx	#<getGibl
	ldy	#>getGibl
	jsr	osCli
	ldx	#<runGibl
	ldy	#>runGibl
	jmp	osCli

:	lda	#1
	rts

getGibl:	.byte	"GET gibl",13
runGibl:	.byte	"/gibl",13

.endproc
